<?php

namespace Drupal\mapsemble\Controller;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\Cache\CacheableResponse;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Render\RenderContext;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\Site\Settings;
use Drupal\Core\Theme\ThemeManager;
use Drupal\mapsemble\Entity\MapsembleMap;
use Drupal\mapsemble\MapsembleApi;
use Symfony\Component\HttpFoundation\RequestStack;
use Drupal\Core\Render\Renderer;
use Drupal\Core\Asset\AssetResolver;
use Drupal\Core\Asset\AttachedAssets;
use Drupal\Core\Asset\LibraryDependencyResolverInterface;
use Drupal\Core\Asset\AssetCollectionRendererInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Controller for bare content without site theming.
 */
abstract class MapsembleContentControllerBase extends ControllerBase {

  /**
   * The request stack.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * The renderer service.
   *
   * @var \Drupal\Core\Render\Renderer
   */
  protected $renderer;

  /**
   * The asset resolver.
   *
   * @var \Drupal\Core\Asset\AssetResolver
   */
  protected $assetResolver;

  /**
   * CSS collection renderer.
   *
   * @var \Drupal\Core\Asset\AssetCollectionRendererInterface
   */
  protected $cssCollectionRenderer;

  /**
   * JS collection renderer.
   *
   * @var \Drupal\Core\Asset\AssetCollectionRendererInterface
   */
  protected $jsCollectionRenderer;

  /**
   * Library dependency resolver.
   *
   * @var \Drupal\Core\Asset\LibraryDependencyResolverInterface
   */
  protected $libraryDependencyResolver;

  /**
   * The theme manager.
   *
   * @var \Drupal\Core\Theme\ThemeManager
   */
  protected $themeManager;

  /**
   * Constructs a new MapsembleContentControllerBase object.
   *
   * @param \Drupal\Core\Render\Renderer $renderer
   *   The renderer service.
   * @param \Drupal\Core\Asset\AssetResolver $assetResolver
   *   The asset resolver.
   * @param \Drupal\Core\Asset\AssetCollectionRendererInterface $cssCollectionRenderer
   *   The CSS collection renderer.
   * @param \Drupal\Core\Asset\AssetCollectionRendererInterface $jsCollectionRenderer
   *   The JS collection renderer.
   * @param \Drupal\Core\Asset\LibraryDependencyResolverInterface $libraryDependencyResolver
   *   The library dependency resolver.
   * @param \Symfony\Component\HttpFoundation\RequestStack $requestStack
   *   The request stack.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   * @param \Drupal\Core\Theme\ThemeManager $themeManager
   *   The theme manager.
   */
  public function __construct(
    Renderer $renderer,
    AssetResolver $assetResolver,
    AssetCollectionRendererInterface $cssCollectionRenderer,
    AssetCollectionRendererInterface $jsCollectionRenderer,
    LibraryDependencyResolverInterface $libraryDependencyResolver,
    RequestStack $requestStack,
    EntityTypeManagerInterface $entityTypeManager,
    ThemeManager $themeManager,
  ) {
    $this->renderer = $renderer;
    $this->assetResolver = $assetResolver;
    $this->cssCollectionRenderer = $cssCollectionRenderer;
    $this->jsCollectionRenderer = $jsCollectionRenderer;
    $this->libraryDependencyResolver = $libraryDependencyResolver;
    $this->requestStack = $requestStack;
    $this->entityTypeManager = $entityTypeManager;
    $this->themeManager = $themeManager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('renderer'),
      $container->get('asset.resolver'),
      $container->get('asset.css.collection_renderer'),
      $container->get('asset.js.collection_renderer'),
      $container->get('library.dependency_resolver'),
      $container->get('request_stack'),
      $container->get('entity_type.manager'),
      $container->get('theme.manager'),
    );
  }

  /**
   * Checks access for the controller.
   *
   * @param \Drupal\Core\Session\AccountProxyInterface $account
   *   The user account.
   *
   * @return \Drupal\Core\Access\AccessResult
   *   The access result.
   */
  public function access(AccountProxyInterface $account) {
    if ($account->hasPermission('administer mapsemble')) {
      return AccessResult::allowed();
    }

    $request = $this->requestStack->getCurrentRequest();
    $authHeader = $request->headers->get('Authorization');
    $bearerToken = NULL;

    if ($authHeader && preg_match('/Bearer\s(\S+)/', $authHeader, $matches)) {
      $bearerToken = $matches[1];
    }

    if ($bearerToken && $bearerToken == $this->config('mapsemble.settings')
      ->get('api_key')) {
      return AccessResult::allowedIfHasPermission($account, 'access content');
    }

    return AccessResult::forbidden();
  }

  /**
   * Gets the view mode for the given map.
   *
   * @param \Drupal\mapsemble\Entity\MapsembleMap $mapsembleMap
   *   The Mapsemble map entity.
   *
   * @return string
   *   The view mode.
   */
  abstract public function getViewMode(MapsembleMap $mapsembleMap): string;

  /**
   * Returns bare content without site theme.
   *
   * @param \Drupal\mapsemble\Entity\MapsembleMap $mapsemble_map
   *   The Mapsemble map entity.
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity to render.
   *
   * @return \Symfony\Component\HttpFoundation\Response
   *   The response object.
   *
   * @throws \Exception
   */
  public function __invoke(MapsembleMap $mapsemble_map, EntityInterface $entity) {
    // Create your build array here.
    $build = [
      '#attached' => [
        'library' => [
          'mapsemble/mapsemble',
        ],
      ],
      'entity' => $this->entityTypeManager->getViewBuilder($entity->getEntityTypeId())
        ->view($entity, $this->getViewMode($mapsemble_map)),
    ];

    $active_theme = $this->themeManager->getActiveTheme();
    $libraries = $active_theme->getLibraries();

    foreach ($libraries as $library) {
      $build['#attached']['library'][] = $library;
    }

    $context = new RenderContext();
    $content = $this->renderer->executeInRenderContext($context, function () use (&$build) {
      return $this->renderer->render($build, TRUE);
    });
    $cacheable_metadata = $context->isEmpty() ? new CacheableMetadata() : $context->pop();

    $attachedAssets = AttachedAssets::createFromRenderArray($build);

    $css_assets = $this->assetResolver->getCssAssets($attachedAssets, FALSE);
    [
      $js_assets_header,
      $js_assets_footer,
    ] = $this->assetResolver->getJsAssets($attachedAssets, FALSE);

    $cssBuild = $this->cssCollectionRenderer->render($css_assets);
    $jsHeaderBuild = $this->jsCollectionRenderer->render($js_assets_header);
    $jsFooterBuild = $this->jsCollectionRenderer->render($js_assets_footer);

    $css = $this->renderer->renderInIsolation($cssBuild);
    $js_header = $this->renderer->renderInIsolation($jsHeaderBuild);
    $js_footer = $this->renderer->renderInIsolation($jsFooterBuild);

    // Build a minimal HTML structure with the assets.
    $html = <<<HTML
<!DOCTYPE html>
<html>
  <head>
    <meta charset="utf-8">
    <meta name="viewport" content="width=device-width, initial-scale=1">
    $css
    $js_header
  </head>
  <body>
    $content
    $js_footer
  </body>
</html>
HTML;

    $domains = [
      Settings::get('mapsemble_embed_base') ?? MapsembleApi::MAPSEMBLE_EMBED_BASE,
      Settings::get('mapsemble_app_base') ?? MapsembleApi::MAPSEMBLE_APP_BASE,
      Settings::get('mapsemble_storefront_base') ?? MapsembleApi::MAPSEMBLE_STOREFRONT_BASE,
    ];

    // Create a custom response.
    $response = new CacheableResponse();
    $response->setContent($html);
    $response->addCacheableDependency($cacheable_metadata);
    $response->headers->set('X-Frame-Options', 'ALLOW-FROM *');
    $response->headers->set('Content-Security-Policy', "frame-ancestors 'self' " . implode(' ', $domains));
    $response->headers->set('Access-Control-Allow-Origin', Settings::get('mapsemble_embed_base') ?? MapsembleApi::MAPSEMBLE_EMBED_BASE);

    return $response;
  }

}
