<?php

namespace Drupal\mapsemble\Controller;

use Drupal\Core\Batch\BatchBuilder;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Url;
use Drupal\mapsemble\Entity\MapsembleMap;
use Symfony\Component\HttpFoundation\RedirectResponse;

/**
 * Controller for Mapsemble batch operations.
 */
class MapsembleController extends ControllerBase {

  /**
   * Synchronizes all entities for a specific map.
   *
   * @param \Drupal\mapsemble\Entity\MapsembleMap $mapsemble_map
   *   The map entity to synchronize.
   *
   * @return \Symfony\Component\HttpFoundation\RedirectResponse
   *   A redirect response to the batch processing page.
   */
  public function syncAllEntities(MapsembleMap $mapsemble_map): RedirectResponse {
    $batch_builder = new BatchBuilder();
    $batch_builder->setTitle($this->t('Synchronizing entities for map @label', ['@label' => $mapsemble_map->label()]))
      ->setInitMessage($this->t('Starting synchronization...'))
      ->setProgressMessage($this->t('Synchronizing entities...'))
      ->setErrorMessage($this->t('An error occurred during synchronization.'))
      ->setFinishCallback(
        [static::class, 'batchFinishedCallback'],
      );

    // Add operations to the batch.
    $entities = $this->getEntitiesForMap($mapsemble_map);
    foreach ($entities as $entity) {
      $batch_builder->addOperation(
        [static::class, 'syncEntityBatch'],
        [$entity->id(), $entity->getEntityTypeId(), $mapsemble_map->id()]
      );
    }

    // Set the batch and redirect to the batch processing page.
    batch_set($batch_builder->toArray());
    return batch_process(Url::fromRoute('entity.mapsemble_map.collection')->toString());
  }

  /**
   * Callback function for batch processing completion.
   *
   * @param bool $success
   *   Indicates if the batch was successful.
   * @param array $results
   *   The results of the batch operations.
   * @param array $operations
   *   The operations that were processed.
   */
  public static function batchFinishedCallback(bool $success, array $results, array $operations): void {
    if ($success) {
      \Drupal::messenger()->addMessage(t('Successfully syncronised content to mapsembls. Processed @count entities.', [
        '@count' => $results['processed'] ?? 0,
      ]));
    }
    else {
      \Drupal::messenger()->addMessage(t('Batch did not complete. Some operations were not finished.'));
    }
  }

  /**
   * Batch operation to synchronize a single entity.
   *
   * @param int $entity_id
   *   The ID of the entity to synchronize.
   * @param string $entity_type_id
   *   The entity type ID.
   * @param string $map_id
   *   The ID of the map.
   * @param array $context
   *   The batch context.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public static function syncEntityBatch(int $entity_id, string $entity_type_id, string $map_id, array &$context): void {
    $entity = \Drupal::entityTypeManager()->getStorage($entity_type_id)->load($entity_id);
    $map = \Drupal::entityTypeManager()->getStorage('mapsemble_map')->load($map_id);

    assert($map instanceof MapsembleMap);
    if ($entity && $map) {
      /** @var \Drupal\mapsemble\MapsembleApi $mapsemble_api */
      $mapsemble_api = \Drupal::service('mapsemble.api');
      $mapsemble_api->syncEntity($entity, $map);
      $context['results']['processed'] = ($context['results']['processed'] ?? 0) + 1;
    }

    $context['message'] = t('Synchronizing entity ID @id for map ID @map_id', [
      '@id' => $entity_id,
      '@map_id' => $map_id,
    ]);
  }

  /**
   * Retrieves entities for a specific map.
   *
   * @param \Drupal\mapsemble\Entity\MapsembleMap $mapsemble_map
   *   The map entity.
   *
   * @return \Drupal\Core\Entity\EntityInterface[]
   *   An array of entities.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  protected function getEntitiesForMap(MapsembleMap $mapsemble_map): array {
    /** @var \Drupal\mapsemble\MapsembleApi $mapsemble_api */
    $mapsemble_api = \Drupal::service('mapsemble.api');
    return $mapsemble_api->getDataEntitiesFromMap($mapsemble_map);
  }

}
