<?php

declare(strict_types=1);

namespace Drupal\mapsemble\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\EntityDisplayRepositoryInterface;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Entity\EntityTypeBundleInfoInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Drupal\Core\Site\Settings;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\Core\Url;
use Drupal\mapsemble\Entity\MapsembleMap;
use Drupal\mapsemble\MapsembleApi;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Map form.
 */
final class MapsembleMapForm extends EntityForm {

  /**
   * The Mapsemble API service.
   *
   * @var \Drupal\mapsemble\MapsembleApi
   */
  protected MapsembleApi $mapsembleApi;

  /**
   * The bundle info service.
   *
   * @var \Drupal\Core\Entity\EntityTypeBundleInfoInterface
   */
  protected EntityTypeBundleInfoInterface $bundleInfo;

  /**
   * The entity display repository service.
   *
   * @var \Drupal\Core\Entity\EntityDisplayRepositoryInterface
   */
  protected EntityDisplayRepositoryInterface $entityDisplayRepository;

  /**
   * The entity field manager service.
   *
   * @var \Drupal\Core\Entity\EntityFieldManagerInterface
   */
  protected EntityFieldManagerInterface $entityFieldManager;

  /**
   * The remote maps.
   *
   * @var array
   */
  protected array $remoteMaps;

  /**
   * Creates an instance of the form.
   *
   * @param \Symfony\Component\DependencyInjection\ContainerInterface $container
   *   The container.
   *
   * @return static
   *   The form instance.
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('entity_type.manager'),
      $container->get('entity_display.repository'),
      $container->get('entity_field.manager'),
      $container->get('entity_type.bundle.info'),
      $container->get('mapsemble.api')
    );
  }

  /**
   * Constructs a new MapsembleMapForm object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Entity\EntityDisplayRepositoryInterface $entity_display_repository
   *   The entity display repository.
   * @param \Drupal\Core\Entity\EntityFieldManagerInterface $entity_field_manager
   *   The entity field manager.
   * @param \Drupal\Core\Entity\EntityTypeBundleInfoInterface $bundle_info
   *   The bundle info service.
   * @param \Drupal\mapsemble\MapsembleApi $mapsemble_api
   *   The Mapsemble API service.
   */
  public function __construct(
    ConfigFactoryInterface $config_factory,
    EntityTypeManagerInterface $entity_type_manager,
    EntityDisplayRepositoryInterface $entity_display_repository,
    EntityFieldManagerInterface $entity_field_manager,
    EntityTypeBundleInfoInterface $bundle_info,
    MapsembleApi $mapsemble_api,
  ) {
    $this->configFactory = $config_factory;
    $this->entityTypeManager = $entity_type_manager;
    $this->entityDisplayRepository = $entity_display_repository;
    $this->entityFieldManager = $entity_field_manager;
    $this->bundleInfo = $bundle_info;
    $this->mapsembleApi = $mapsemble_api;
  }

  /**
   * {@inheritdoc}
   */
  public function actions(array $form, FormStateInterface $form_state) {
    if (isset($form['id'])) {
      $actions = parent::actions($form, $form_state);
      if ($this->entity->isNew()) {
        $actions['submit']['#value'] = $this->t('Create and configure');
      }

      return $actions;
    }
    else {
      return $form;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state): array {
    assert($this->entity instanceof MapsembleMap);
    $form = parent::form($form, $form_state);

    $accessToken = $this->configFactory->get('mapsemble.settings')->get(
      'api_key'
    );
    if (!$accessToken) {
      $accessTokenlink = Link::fromTextAndUrl(
        new TranslatableMarkup('here'),
        Url::fromRoute('mapsemble.settings')
      );
      $createlink = Link::fromTextAndUrl(
        new TranslatableMarkup('create a map'),
        Url::fromUri(
          (Settings::get(
            'mapsemble_app_base'
          ) ?? MapsembleApi::MAPSEMBLE_APP_BASE) . '/integrations/drupal',
          [
            'attributes' => [
              'target' => '_blank',
            ],
          ]
        )
      );
      $this->messenger()->addWarning(
        $this->t(
          'Please @create_link  with a Drupal layer in Mapsemble first. Also configure your api key @api_key_link.',
          [
            '@create_link' => $createlink->toString(),
            '@api_key_link' => $accessTokenlink->toString(),
          ]
        )
      );

      return $form;
    }
    else {
      try {
        $this->remoteMaps = $this->mapsembleApi->getRemoteMaps();
      }
      catch (\Exception $e) {
        $this->messenger()->addError(
          $this->t(
            'Error fetching remoteMaps from Mapsemble. Error: @error',
            ['@error' => $e->getMessage()]
          )
        );

        return $form;
      }

      if (!$this->remoteMaps) {
        $link = Link::fromTextAndUrl(
          new TranslatableMarkup('create a map'),
          Url::fromUri(
            (Settings::get(
              'mapsemble_app_base'
            ) ?? MapsembleApi::MAPSEMBLE_APP_BASE) . '/integrations/drupal',
            [
              'attributes' => [
                'target' => '_blank',
              ],
            ]
          )
        );
        $this->messenger()->addWarning(
          $this->t(
            'No remoteMaps found with a Drupal layer. First @link in Mapsemble with a Drupal layer. If necessary, create an account first.',
            [
              '@link' => $link->toString(),
            ]
          )
        );

        return $form;
      }
    }
    $form['label'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Label'),
      '#maxlength' => 255,
      '#default_value' => $this->entity->label(),
      '#required' => TRUE,
    ];

    $form['id'] = [
      '#type' => 'machine_name',
      '#machine_name' => [
        'exists' => [MapsembleMap::class, 'load'],
      ],
      '#disabled' => !$this->entity->isNew(),
      '#default_value' => $this->entity->id(),
    ];

    $form['messages'] = [
      '#theme' => 'status_messages',
      '#message_list' => [],
    ];

    $form['mapsemble'] = [
      '#type' => 'details',
      '#title' => $this->t('Mapsemble'),
      '#open' => TRUE,
    ];

    $form['mapsemble']['mapId'] = [
      '#type' => 'select',
      '#title' => $this->t('Map'),
      '#options' => array_map(fn($map) => $map['label'], $this->remoteMaps),
      '#default_value' => $this->entity->get('mapId'),
      '#ajax' => [
        'callback' => '::updateLayerOptions',
        'wrapper' => 'layer-wrapper',
      ],
      '#empty_option' => $this->t('- Select -'),
      '#required' => TRUE,

    ];

    $form['mapsemble']['#prefix'] = '<div id="layer-wrapper">';
    $form['mapsemble']['#suffix'] = '</div>';

    $layerOptions = $this->getLayerOptions(
      $this->remoteMaps,
      $this->entity->get('mapId')
    );

    if (!in_array($this->entity->get('layerId'), array_keys($layerOptions))) {
      $this->entity->set('layerId', '');
    }

    $form['mapsemble']['layerId'] = [
      '#type' => 'select',
      '#title' => $this->t('Layer'),
      '#default_value' => in_array($this->entity->get('layerId'), array_keys($layerOptions)) ? $this->entity->get('layerId') : '',
      '#required' => TRUE,
      '#options' => $layerOptions,
      '#access' => $form_state->getValue('mapId') || $this->entity->get(
          'mapId'
      ),
      '#empty_option' => $this->t('- Select -'),
    ];

    $form['settings'] = [
      '#type' => 'details',
      '#prefix' => '<div id="mapsemble-map-configure-form">',
      '#suffix' => '</div>',
      '#title' => $this->t('Drupal'),
      '#open' => TRUE,
      '#tree' => TRUE,
      'entity_type_id' => [
        '#type' => 'select',
        '#title' => $this->t('Entity type'),
        '#default_value' => $this->entity->get('settings')['entity_type_id'] ?? '',
        '#options' => $this->getEntityTypeOptions(),
        '#ajax' => [
          'callback' => '::refreshEntityConfig',
          'wrapper' => 'mapsemble-map-configure-form',
        ],
        '#empty_option' => $this->t('- Select -'),
        '#required' => TRUE,
      ],
    ];

    $entity_type_id = $form_state->getValue([
      'settings',
      'entity_type_id',
    ]) ?? $this->entity->get('settings')['entity_type_id'] ?? '';

    $viewModes = $this->entityDisplayRepository->getViewModes($entity_type_id);
    $form['settings']['bundle'] = [
      '#type' => 'select',
      '#title' => $this->t('Bundle'),
      '#default_value' => $this->entity->get('settings')['bundle'] ?? '',
      '#options' => array_map(function ($bundle) {
        return $bundle['label'];
      }, $this->bundleInfo->getBundleInfo($entity_type_id)),
      '#ajax' => [
        'callback' => '::refreshEntityConfig',
        'wrapper' => 'mapsemble-map-configure-form',
      ],
      '#empty_option' => $this->t('- Select -'),
      '#access' => (bool) $entity_type_id,
      '#required' => TRUE,
    ];

    $bundle = $form_state->getValue([
      'settings',
      'bundle',
    ]) ?? $this->entity->get('settings')['bundle'] ?? '';

    $fields = [];
    if ($bundle) {
      $fields = $this->entityFieldManager->getFieldDefinitions(
        $entity_type_id,
        $bundle
      );
      foreach ($fields as $field) {
        if ($field->getType() !== 'geofield') {
          unset($fields[$field->getName()]);
        }
      }
    }

    if ($bundle && empty($fields)) {
      $this->messenger()->addWarning(
        $this->t(
          'No geofield found for the selected entity type and bundle. Please select a different entity.'
        )
      );
    }

    $form['settings']['location_field'] = [
      '#type' => 'select',
      '#title' => $this->t('Location field'),
      '#default_value' => $this->entity->get('settings')['location_field'] ?? '',
      '#ajax' => [
        'callback' => '::refreshEntityConfig',
        'wrapper' => 'mapsemble-map-configure-form',
      ],
      '#options' => array_map(
        fn($field) => $field->label(),
        $fields ?? []
      ),
      '#empty_option' => $this->t('- Select -'),
      '#required' => TRUE,
      '#access' => (bool) $bundle,
    ];

    $location_field = $form_state->getValue([
      'settings',
      'location_field',
    ]) ?? $this->entity->get('settings')['location_field'] ?? '';

    $form['settings']['popup_view_mode'] = [
      '#type' => 'select',
      '#title' => $this->t('Popup view mode'),
      '#description' => $this->t('The view mode to use for the popup in Mapsemble.'),
      '#default_value' => $this->entity->get('settings')['popup_view_mode'] ?? '',
      '#options' => array_map(
        fn($view_mode) => $view_mode['label'],
        $viewModes
      ),
      '#empty_option' => $this->t('- Select -'),
      '#required' => TRUE,
      '#access' => (bool) $location_field,
    ];
    $form['settings']['card_view_mode'] = [
      '#type' => 'select',
      '#title' => $this->t('Card view mode'),
      '#description' => $this->t('The view mode to use for the card view in Mapsemble.'),
      '#default_value' => $this->entity->get('settings')['card_view_mode'] ?? '',
      '#options' => array_map(
        fn($view_mode) => $view_mode['label'],
        $viewModes
      ),
      '#empty_option' => $this->t('- Select -'),
      '#required' => TRUE,
      '#access' => (bool) $location_field,
    ];
    $form['settings']['data_view_mode'] = [
      '#type' => 'select',
      '#title' => $this->t('Data view mode'),
      '#default_value' => $this->entity->get('settings')['data_view_mode'] ?? '',
      '#options' => array_map(
        fn($view_mode) => $view_mode['label'],
        $viewModes
      ),
      '#empty_option' => $this->t('- Select -'),
      '#required' => TRUE,
      '#access' => (bool) $location_field,
    ];

    return $form;
  }

  /**
   * Updates the layer options based on the selected map.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   *
   * @return array
   *   The updated form array.
   */
  public function updateLayerOptions(
    array $form,
    FormStateInterface $form_state,
  ): array {
    $maps = $this->mapsembleApi->getRemoteMaps();
    $mapId = $form_state->getValue('mapId');
    $form_state->set(['mapsemble', 'layerId'], '');
    $form['mapsemble']['layer']['#options'] = $this->getLayerOptions(
      $maps,
      $mapId
    );
    $form['mapsemble']['layer']['#default_value'] = '';

    return $form['mapsemble'];
  }

  /**
   * Gets the layer options for a given map.
   *
   * @param array $maps
   *   The maps array.
   * @param string $mapId
   *   The map ID.
   *
   * @return array
   *   The layer options.
   */
  public function getLayerOptions(array $maps, ?string $mapId = NULL): array {
    if (!$mapId) {
      return [];
    }
    $map = FALSE;
    foreach ($maps as $existingMap) {
      if ($existingMap['id'] === $mapId) {
        $map = $existingMap;
      }
    }
    if (!$map) {
      return [];
    }
    $layers = [];

    foreach ($map['layers'] as $layer) {
      if ($layer['type'] === 'vector:drupal') {
        $layers[$layer['id']] = $layer['label'];
      }
    }

    return $layers;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $layerId = $form_state->getValue('layerId');
    foreach ($this->remoteMaps as $map) {
      if ($map['id'] === $form_state->getValue('mapId')) {
        foreach ($map['layers'] as $layer) {
          if ($layer['id'] === $layerId) {
            $this->entity->set('dataSourceId', $layer['dataSource']['id']);
            break 2;
          }
        }
      }
    }
    parent::submitForm(
      $form,
      $form_state
    );
  }

  /**
   * Gets the entity type options.
   *
   * @return array
   *   The entity type options.
   */
  protected function getEntityTypeOptions() {
    $options = [];
    foreach (
      $this->entityTypeManager->getDefinitions() as $entity_type_id => $entity_type
    ) {
      if (
        $entity_type->entityClassImplements(
          ContentEntityInterface::class
        )
      ) {
        $options[$entity_type_id] = $entity_type->getLabel();
      }
    }

    return $options;
  }

  /**
   * Reloads the configuration form.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   *
   * @return array
   *   The updated form array.
   */
  public function refreshEntityConfig(array $form, FormStateInterface $form_state) {
    return $form['settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state): int {
    $result = parent::save($form, $form_state);
    $message_args = ['%label' => $this->entity->label()];
    $this->messenger()->addStatus(
      match ($result) {
        \SAVED_NEW => $this->t('Created new example %label.', $message_args),
        \SAVED_UPDATED => $this->t('Updated example %label.', $message_args),
      }
    );
    $form_state->setRedirectUrl($this->entity->toUrl('collection'));

    return $result;
  }

}
